import { SUBTLE, PAGODA, LAVA, AZALE, METRO } from '@palett/presets';
import { DecoObject } from '@spare/deco-object';
import { dateTime } from '@valjoux/timestamp-pretty';
import { DecoString } from '@spare/deco-string';
import { DecoVector } from '@spare/deco-vector';
import { sync } from 'brotli-size';
import fileSize from 'filesize';
import { gzipSizeSync } from 'gzip-size';
import { minify } from 'terser';

/** @type {function} */
const decoFileName = DecoString({ thres: NaN, pres: { str: SUBTLE, num: PAGODA } });
/** @type {function} */
const decoNames = DecoVector({ thres: NaN, delim: '/', pres: { str: SUBTLE, num: LAVA } });
/** @type {function} */
const decoSizes = DecoVector({ thres: NaN, delim: '/', pres: { str: AZALE, num: PAGODA } });

/**
 *
 * @param {OutputChunk} bundle
 * @param {Object} p
 * @param {Object} [p.format]
 * @param {Function} [p.render]
 * @param {Object} [p.preset]
 * @param {boolean} [p.showGzipped=true]
 * @param {boolean} [p.showBrotli=true]
 * @param {boolean} [p.showMinified=true]
 * @return {Object}
 */
const sizeInfo = async function (bundle, p) {
  const { code, fileName } = bundle, { format } = p;
  const minifiedCode = (await minify(code))?.code ?? '';
  const info = {};
  const sizes = { bundle: fileSize(Buffer.byteLength(code), format) };
  if (p.showBrotli) Object.assign(sizes, { brotli: fileSize(sync(code), format) });
  if (p.showMinified) Object.assign(sizes, { min: fileSize(minifiedCode.length, format) });
  if (p.showGzipped) Object.assign(sizes, { gzip: fileSize(gzipSizeSync(minifiedCode), format) });
  info['file'] = decoFileName(fileName);
  info[decoNames(Object.keys(sizes))] = decoSizes(Object.values(sizes), { pres: p.preset });
  return p.render ? p.render(info) : info
};

const decoSizeInfoObject = DecoObject({ thres: NaN });

const fileInfo = (config = {}) => {
  const defaultConfig = {
    format: {},
    preset: METRO,
    render: o => `${dateTime()} ${decoSizeInfoObject(o)}`,
    showGzipped: true,
    showBrotli: false,
    showMinified: true
  };
  config = Object.assign(defaultConfig, config);
  return {
    name: 'fileInfo',
    generateBundle(options, bundle, isWrite) {
      Object
        .values(bundle)
        .filter(({ type }) => type !== 'asset')
        .forEach(async subBundle => console.log(await sizeInfo(subBundle, config))); // console.log(miscInfo(subBundle))
    }
  }
};

export { fileInfo as default, fileInfo };
